@@ -1,9 +1,9 @@
-// $Id: sp_log.cpp 11992 2006-01-16 18:59:58Z wsnyder $
+// $Id: sp_log.cpp 37619 2007-04-30 13:20:11Z wsnyder $
 //********************************************************************
 //
 // THIS MODULE IS PUBLICLY LICENSED
 //
-// Copyright 2001-2006 by Wilson Snyder.  This program is free software;
+// Copyright 2001-2007 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
 // General Public License or the Perl Artistic License.
 //
@@ -60,6 +60,9 @@ void sp_log_file::close() {
 
 void sp_log_file::close_int() {
     if (m_isOpen) {
+        if (!this->flush()) {
+            std::cerr <<"%Error: sp_log_file:close_int(): flusing writes of '" <<m_filename<<"'"<<endl;
+        }
 	m_isOpen = false;
 	remove_file();
 	std::ofstream::close();
